Package: SplicingGraphs
Title: Create, manipulate, visualize splicing graphs, and assign
        RNA-seq reads to them
Description: This package allows the user to create, manipulate, and visualize
	splicing graphs and their bubbles based on a gene model for a given
	organism. Additionally it allows the user to assign RNA-seq reads to
	the edges of a set of splicing graphs, and to summarize them in
	different ways.
biocViews: Genetics, Annotation, DataRepresentation, Visualization,
        Sequencing, RNASeq, GeneExpression, AlternativeSplicing,
        Transcription, ImmunoOncology
URL: https://bioconductor.org/packages/SplicingGraphs
BugReports: https://github.com/Bioconductor/SplicingGraphs/issues
Version: 1.53.0
License: Artistic-2.0
Encoding: UTF-8
Author: D. Bindreither, M. Carlson, M. Morgan, H. Pagès
Maintainer: H. Pagès <hpages.on.github@gmail.com>
Depends: R (>= 3.5.0), GenomicFeatures (>= 1.17.13), GenomicAlignments
        (>= 1.1.22), Rgraphviz (>= 2.3.7)
Imports: methods, utils, graphics, igraph, BiocGenerics, S4Vectors (>=
        0.17.5), BiocParallel, IRanges (>= 2.21.2), Seqinfo,
        GenomicRanges (>= 1.23.21), Rsamtools, graph
Suggests: igraph, Gviz, txdbmaker, TxDb.Hsapiens.UCSC.hg19.refGene,
        RNAseqData.HNRNPC.bam.chr14, RUnit
Collate: utils.R igraph-utils.R SplicingGraphs-class.R
        plotTranscripts-methods.R sgedgesByGene-methods.R
        txpath-methods.R sgedges-methods.R sgraph-methods.R
        rsgedgesByGene-methods.R bubbles-methods.R assignReads.R
        countReads-methods.R toy_data.R zzz.R
git_url: https://git.bioconductor.org/packages/SplicingGraphs
git_branch: devel
git_last_commit: 0d0f030
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:24:58 UTC; biocbuild
